@@ -51,20 +51,19 @@ def do_train(cfg, arguments,
     max_iter = (max_epoch - start_epoch) * epoch_iters
     current_iterations = 0
 
-    if cfg.TRAIN.HYBRID_PRECISION:
-        # Creates a GradScaler once at the beginning of training.
-        scaler = GradScaler()
+    # Creates a GradScaler once at the beginning of training.
+    scalar = GradScaler()
+    model.train()
+    optimizer.zero_grad()
 
     synchronize()
-    model.train()
     logger.info("Start training ...")
     # Perform the training loop.
     logger.info("Start epoch: {}".format(start_epoch))
     start_training_time = time.time()
     end = time.time()
     for cur_epoch in range(start_epoch, max_epoch + 1):
-        if cfg.DATALOADER.SHUFFLE:
-            shuffle_dataset(train_data_loader, cur_epoch)
+        shuffle_dataset(train_data_loader, cur_epoch, is_shuffle=cfg.DATALOADER.RANDOM_SAMPLE)
         data_loader = Prefetcher(train_data_loader, device) if cfg.DATALOADER.PREFETCHER else train_data_loader
         for iteration, (images, targets) in enumerate(data_loader):
             if not cfg.DATALOADER.PREFETCHER:
@@ -77,39 +76,33 @@ def do_train(cfg, arguments,
                     output_dict = model(images)
                     loss_dict = criterion(output_dict, targets)
                     loss = loss_dict[KEY_LOSS] / gradient_accumulate_step
-
-                current_iterations += 1
-                if current_iterations % gradient_accumulate_step != 0:
-                    if isinstance(model, DistributedDataParallel):
-                        # multi-gpu distributed training
-                        with model.no_sync():
-                            scaler.scale(loss).backward()
-                    else:
-                        scaler.scale(loss).backward()
+                if isinstance(model, DistributedDataParallel):
+                    # multi-gpu distributed training
+                    with model.no_sync():
+                        scalar.scale(loss).backward()
                 else:
-                    scaler.scale(loss).backward()
-                    scaler.step(optimizer)
-                    scaler.update()
-                    current_iterations = 0
-                    optimizer.zero_grad()
+                    scalar.scale(loss).backward()
             else:
                 output_dict = model(images)
                 loss_dict = criterion(output_dict, targets)
                 loss = loss_dict[KEY_LOSS] / gradient_accumulate_step
 
-                current_iterations += 1
-                if current_iterations % gradient_accumulate_step != 0:
-                    if isinstance(model, DistributedDataParallel):
-                        # multi-gpu distributed training
-                        with model.no_sync():
-                            loss.backward()
-                    else:
+                if isinstance(model, DistributedDataParallel):
+                    # multi-gpu distributed training
+                    with model.no_sync():
                         loss.backward()
                 else:
                     loss.backward()
+
+            current_iterations += 1
+            if current_iterations % gradient_accumulate_step == 0:
+                current_iterations = 0
+                if cfg.TRAIN.HYBRID_PRECISION:
+                    scalar.step(optimizer)
+                    scalar.update()
+                else:
                     optimizer.step()
-                    current_iterations = 0
-                    optimizer.zero_grad()
+                optimizer.zero_grad()
 
             acc_list = evaluator.evaluate_train(output_dict, targets)
             update_stats(cfg.NUM_GPUS, meters, loss_dict, acc_list)
@@ -131,6 +124,8 @@ def do_train(cfg, arguments,
 
         if cfg.DATALOADER.PREFETCHER:
             data_loader.release()
+            del data_loader
+        torch.cuda.empty_cache()
         logger.info(log_epoch_stats(epoch_iters, cur_epoch, max_epoch, optimizer.param_groups[0]['lr'], meters))
         arguments["cur_epoch"] = cur_epoch
         lr_scheduler.step()
